package com.example.audiotrack

import android.content.Context
import android.graphics.*
import android.graphics.drawable.Drawable
import android.support.v4.content.ContextCompat
import android.support.v4.view.GestureDetectorCompat
import android.util.AttributeSet
import android.view.GestureDetector
import android.view.MotionEvent
import android.view.View

abstract class AudioTrackView : View, GestureDetector.OnGestureListener {
    interface OnScrollListener {
        fun onScroll(scrollRatio: Double)
    }

    var outerLeft: Double = 0.0
    var outerRight: Double = width.toDouble()

    var startX: Double = 0.0
    var startY: Double = 0.0

    var innerWidthRatio: Float = 0f
    var innerWidth: Double = 0.0
    var innerColor: Int = 0

    var drawInnerColor:Int = 0

    var endDrawable: Drawable? = null

    var scrollable: Boolean = false
    internal var onScrollListener: OnScrollListener? = null

    protected val paint = Paint()

    private val body: RectF = RectF()

    private val gestureDetectorCompat = GestureDetectorCompat(context, this)


    constructor(context: Context) : super(context)

    constructor(context: Context, attributeSet: AttributeSet) : super(context, attributeSet) {
        initAttr(attributeSet)
    }

    constructor(context: Context, attributeSet: AttributeSet, defStyleAttr: Int) : super(context, attributeSet, defStyleAttr) {
        initAttr(attributeSet)
    }

    private fun initAttr(attributeSet: AttributeSet) {
        val typedArray = context.theme.obtainStyledAttributes(attributeSet, R.styleable.AudioTrackView, 0, 0)
        innerWidthRatio = typedArray.getFloat(R.styleable.AudioTrackView_innerWidthRatio, 0f)
        innerColor = typedArray.getColor(R.styleable.AudioTrackView_innerColor, ContextCompat.getColor(context, android.R.color.white))
        endDrawable = typedArray.getDrawable(R.styleable.AudioTrackView_endDrawable)

        drawInnerColor = innerColor
    }

    init {
        paint.isAntiAlias = true
    }

    fun setOnScrollsListener(onScrollListener: OnScrollListener) {
        this.onScrollListener = onScrollListener
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)

        canvas?.let {
            drawBackground(canvas, outerLeft.toInt(), outerRight.toInt())
            drawBody(canvas, startX, startY)
        }
    }

    open fun drawBackground(canvas: Canvas, left: Int, right: Int) {
        background.bounds = Rect(left, 0, right, height)
        background.draw(canvas)
    }

    private fun drawBody(canvas: Canvas, drawX: Double, drawY: Double) {
        val drawEndX = if ((drawX + innerWidth) > outerRight) outerRight else drawX + innerWidth
        val drawEndY = drawY + height

        paint.color = drawInnerColor
        body.set(drawX.toFloat(), drawY.toFloat(), drawEndX.toFloat(), drawEndY.toFloat())
        canvas.drawRect(body, paint)
    }

    open fun drawEnd(canvas: Canvas, drawX: Double, drawY: Double, height: Int) {
        endDrawable?.let {
            val drawCenterX = if ((drawX + innerWidth) > outerRight) outerRight else drawX + innerWidth
            val drawCenterY = drawY + height / 2
            val drawRadius = it.intrinsicWidth / 2
            val shadowRange = resources.getDimension(R.dimen.shadow_range)

            paint.color = ContextCompat.getColor(context, R.color.shadow_layer2)
            canvas.drawCircle(drawCenterX.toFloat(),
                    (drawCenterY + 2 * shadowRange).toFloat(),
                    drawRadius + 2 * shadowRange,
                    paint)

            paint.color = ContextCompat.getColor(context, R.color.shadow_layer1)
            canvas.drawCircle(drawCenterX.toFloat(),
                    (drawCenterY + shadowRange).toFloat(),
                    drawRadius + shadowRange,
                    paint)

            paint.color = ContextCompat.getColor(context, android.R.color.white)
            canvas.drawCircle(drawCenterX.toFloat(), drawCenterY.toFloat(), drawRadius.toFloat(), paint)

            it.bounds = Rect(
                    (drawCenterX - drawRadius).toInt(),
                    (drawCenterY - drawRadius).toInt(),
                    (drawCenterX + drawRadius).toInt(),
                    (drawCenterY + drawRadius).toInt())
            it.draw(canvas)
        }
    }

    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)

        endDrawable?.let {
            outerLeft = (it.intrinsicWidth / 2).toDouble()
            outerRight = (width - it.intrinsicWidth / 2).toDouble()
        }

        startX = outerLeft

        innerWidth = (innerWidthRatio * (outerRight - outerLeft))
    }

    override fun onTouchEvent(event: MotionEvent?): Boolean {
        if (gestureDetectorCompat.onTouchEvent(event)) {
            return true
        }
        return super.onTouchEvent(event)
    }

    override fun onDown(p0: MotionEvent?): Boolean {
        var endX = startX + innerWidth
        val endY = startY + height

        endDrawable?.let {
            endX -= it.intrinsicWidth / 2
        }

        p0?.let {
            return (it.x in startX..endX) && (it.y in startY..endY)
        }
        return false
    }

    override fun onScroll(p0: MotionEvent?, p1: MotionEvent?, p2: Float, p3: Float): Boolean {
        if (scrollable) {

            startX = if (startX - p2 < outerLeft) outerLeft
            else if (startX - p2 > (outerRight - innerWidth)) outerRight - innerWidth
            else startX - p2

            onScrollListener?.onScroll((startX - outerLeft) / (outerRight - outerLeft))

            invalidate()
            return true
        }
        return false
    }

    override fun onSingleTapUp(p0: MotionEvent?): Boolean {
        return true
    }

    override fun onShowPress(p0: MotionEvent?) {
    }

    override fun onFling(p0: MotionEvent?, p1: MotionEvent?, p2: Float, p3: Float): Boolean {
        return false
    }

    override fun onLongPress(p0: MotionEvent?) {
    }
}