package com.example.audiotrack

import android.content.Context
import android.graphics.Canvas
import android.graphics.RectF
import android.os.Build
import android.support.v4.content.ContextCompat
import android.text.Layout
import android.text.StaticLayout
import android.text.TextPaint
import android.text.TextUtils
import android.util.AttributeSet
import android.view.MotionEvent

class LoopAudioTrackView : AudioTrackView {
    interface OnLoopScrollListener {
        fun onScroll(scrollRatio: Double)
    }

    var innerText: String = ""
    var innerTextSize: Int = 0
    var innerTextPadding: Int = 0

    var isSelect: Boolean = false

    var isLoop: Boolean = false
    private var loopWidth: Int = 0
    private var onLoopScrollListener: OnLoopScrollListener? = null

    private val loop: RectF = RectF()

    private val textPaint = TextPaint()

    companion object {
        const val PAINT_MAX_ALPHA = 255
    }

    constructor(context: Context) : super(context)

    constructor(context: Context, attributeSet: AttributeSet) : super(context, attributeSet) {
        initAttr(attributeSet)
    }

    constructor(context: Context, attributeSet: AttributeSet, defStyleAttr: Int) : super(context, attributeSet, defStyleAttr) {
        initAttr(attributeSet)
    }

    private fun initAttr(attributeSet: AttributeSet) {
        val typedArray = context.theme.obtainStyledAttributes(attributeSet, R.styleable.LoopAudioTrackView, 0, 0)
        innerText = typedArray.getString(R.styleable.LoopAudioTrackView_innerText)
        innerTextSize = typedArray.getDimensionPixelSize(R.styleable.LoopAudioTrackView_innerTextSize, resources.getDimensionPixelSize(R.dimen.default_inner_text_size))
        innerTextPadding = typedArray.getDimensionPixelSize(R.styleable.LoopAudioTrackView_innerTextPadding, 0)
        isLoop = typedArray.getBoolean(R.styleable.LoopAudioTrackView_isLoop, false)
    }

    init {
        scrollable = true
    }

    fun setOnLoopScrollListener(onLoopScrollListener: OnLoopScrollListener) {
        this.onLoopScrollListener = onLoopScrollListener
    }

    override fun onDraw(canvas: Canvas?) {
        drawInnerColor = if (isSelect) innerColor else ContextCompat.getColor(context, android.R.color.darker_gray)

        super.onDraw(canvas)

        canvas?.let {
            drawText(canvas, startX, startY)

            if (isLoop) {
                drawLoop(canvas, startX, startY)
                drawEnd(canvas, startX + loopWidth, startY, height)
            }
        }
    }

    private fun drawText(canvas: Canvas, drawX: Double, drawY: Double) {
        var ellipsizedWidth = innerWidth
        endDrawable?.let {
            ellipsizedWidth = innerWidth - it.intrinsicWidth / 2
        }

        textPaint.isAntiAlias = true
        textPaint.textSize = innerTextSize.toFloat()
        textPaint.color = ContextCompat.getColor(context, android.R.color.black)

        val staticLayout: StaticLayout = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            val staticLayoutBuilder = StaticLayout.Builder.obtain(innerText, 0, innerText.length, textPaint, innerWidth.toInt())
            staticLayoutBuilder.setEllipsizedWidth(ellipsizedWidth.toInt())
            staticLayoutBuilder.setEllipsize(TextUtils.TruncateAt.END)
            staticLayoutBuilder.setMaxLines(1)
            staticLayoutBuilder.build()
        } else {
            StaticLayout(innerText,
                    0,
                    innerText.length,
                    textPaint,
                    innerWidth.toInt(),
                    Layout.Alignment.ALIGN_NORMAL,
                    0f,
                    0f,
                    true,
                    TextUtils.TruncateAt.END,
                    ellipsizedWidth.toInt())
        }

        if (staticLayout.width >= resources.getDimensionPixelSize(R.dimen.inner_text_display_min_width)) {
            val drawTextX = drawX + innerTextPadding
            val drawTextY = drawY + height / 2 - staticLayout.height / 2

            canvas.save()
            canvas.translate(drawTextX.toFloat(), drawTextY.toFloat())
            staticLayout.draw(canvas)
            canvas.restore()
        }
    }

    private fun drawLoop(canvas: Canvas, drawX: Double, drawY: Double) {
        val drawStartX = drawX + innerWidth
        val drawEndX = if (drawStartX + loopWidth > outerRight) outerRight else drawStartX + loopWidth

        if (drawStartX < drawEndX) {
            paint.color = drawInnerColor
            paint.alpha = (PAINT_MAX_ALPHA * .6).toInt()
            loop.set(drawStartX.toFloat(), drawY.toFloat(), drawEndX.toFloat(), (drawY + height).toFloat())
            canvas.drawRect(loop, paint)
        }
    }

    override fun onDown(p0: MotionEvent?): Boolean {
        return (isLoop && onDownLoopTrack(p0)) || super.onDown(p0)
    }

    private fun onDownLoopTrack(p0: MotionEvent?): Boolean {
        endDrawable?.let {
            val loopStartX = startX + innerWidth
            val loopEndX = (startX + innerWidth + loopWidth + it.intrinsicWidth / 2
                    + resources.getDimensionPixelSize(R.dimen.extra_touch_event_range))

            p0?.let {
                return ((it.x in loopStartX..loopEndX) && (it.y in startY..(startY + height)))
            }
        }
        return false
    }

    override fun onSingleTapUp(p0: MotionEvent?): Boolean {
        isSelect = isSelect.not()
        invalidate()
        return super.onSingleTapUp(p0)
    }

    override fun onScroll(p0: MotionEvent?, p1: MotionEvent?, p2: Float, p3: Float): Boolean {
        return isSelect && ((isLoop && (onScrollEndDrawable(p1, p2) || onScrollLoopTrack(p2))) || super.onScroll(p0, p1, p2, p3))
    }

    private fun onScrollEndDrawable(p1: MotionEvent?, p2: Float): Boolean {
        val drawableX = if (startX + innerWidth + loopWidth > outerRight) outerRight else startX + innerWidth + loopWidth
        var drawableStartX = 0.0
        var drawableEndX = 0.0

        endDrawable?.let {
            drawableStartX = (drawableX - it.intrinsicWidth / 2 - resources.getDimensionPixelSize(R.dimen.extra_touch_event_range))
            drawableEndX = (drawableX + it.intrinsicWidth / 2 + resources.getDimensionPixelSize(R.dimen.extra_touch_event_range))
        }

        p1?.let {
            if ((it.x in drawableStartX..drawableEndX) && (it.y in startY..(startY + height))) {
                loopWidth = if (loopWidth - p2 < 0) 0
                else if (loopWidth - p2 > (outerRight - (startX + innerWidth))) ((outerRight - (startX + innerWidth)).toInt())
                else (loopWidth - p2).toInt()

                onLoopScrollListener?.onScroll(loopWidth / (outerRight - outerLeft))

                invalidate()

                return true
            }
        }
        return false
    }

    private fun onScrollLoopTrack(p2: Float): Boolean {
        if (loopWidth > 0) {
            startX = if (startX - p2 < outerLeft) outerLeft
            else if (startX - p2 > (outerRight - innerWidth - loopWidth)) outerRight - innerWidth - loopWidth
            else startX - p2

            onScrollListener?.onScroll((startX - outerLeft) / (outerRight - outerLeft))

            invalidate()

            return true
        }
        return false
    }
}